//*******************************************
//
//  Système eruche MRKFOX 1200 installe au Joug
//
// S.GOUTTEBROZE (c) 11.2017
//*******************************************


#include "HX711.h"
#include <OneWire.h>
#include <DHT.h>
#include <SigFox.h>
#include <ArduinoLowPower.h>


//Constants
#define DHTPIN 4     // what pin we're connected to
#define DHTTYPE DHT22   // DHT 22  (AM2302)
#define SLEEPTIME     60 * 60 * 1000   // Set the delay to 60 minutes (60 min x 60 seconds x 1000 milliseconds)
//#define SLEEPTIME     15 * 60 * 1000   // Set the delay to 15 minutes



DHT dht(DHTPIN, DHTTYPE); //// Initialize DHT sensor for normal 16mhz Arduino

// DS18S20 Temperature chip i/o
OneWire ds(2);  // on pin ...


//Variables

 typedef struct __attribute__ ((packed)) sigfox_message { 
  uint16_t t_arduino;
  uint16_t t_int1;
  uint16_t t_int2;
  uint16_t hum;
  uint8_t poids_kg;
  uint16_t poids_g;
  uint8_t bat;
  } SigfoxMessage;

// stub for message which will be sent

SigfoxMessage msgsgfox;



float Vf(0);  // pour la batterie

int chk;
float hum;  //Stores humidity value
float temp; //Stores temperature value


uint8_t poids_kg;
uint16_t poids_g;
uint8_t bat;

 // the setup function runs once when you press reset or power the board
unsigned long Weight = 0;
unsigned long AverageWeight = 0;
unsigned long AverageWeightmoy = 0;

int Clock = 5; // on board
int Dout = 6; // on board
int cpt(0);
int cptP(0);

float poidsD(0);

float Tc_100_A,Tc_100_B;
byte i;
byte type_s;
byte present = 0;
byte data[12];
byte addr[8];
char *msg ;

HX711 scaleA(A2, A1);    // parameter "gain" is ommited; the default value 128 is used by the library




void setup() {
     
  //Serial.begin(9600);
  //Serial.println("begin...");
  
  //Serial.println("Initialize HX711_ modules");
  scaleA.set_scale(20.);                      // this value is obtained by calibrating the scale with known weights; see the README for details

  analogReadResolution(12);
  analogReference(AR_INTERNAL1V0);    
    
  dht.begin();
  
 
  
  // Initialisation...
Tc_100_A=15.2;
Tc_100_B=19.7;

poidsD=15785;

  if (!SigFox.begin()) {
        // Something is really wrong, try rebooting
        // Reboot is useful if we are powering the board using an unreliable power source
        // (eg. solar panels or other energy harvesting methods)
        //Serial.println("reboot...");
        reboot();
  }

 // on initialise avec des valeurs reconnnaissables pour tester la connexion Sigfox et le fonctionnement
  //__builtin_bswap16 == pour passer de little_endian en big_endian   (most significant bit first)
  msgsgfox.t_arduino =  __builtin_bswap16(1.0);
  msgsgfox.t_int1 =  __builtin_bswap16(2.0);
  msgsgfox.t_int2 =  __builtin_bswap16(3.0);
  msgsgfox.hum =  __builtin_bswap16(4.0);
  msgsgfox.poids_kg = 5.0;
  msgsgfox.poids_g = __builtin_bswap16(6.0+1);
  msgsgfox.bat = 7.0;
   
  transmission_sigfox();

delay(60000);

}


void loop() {


// Pour les tests sans Bluetooth,

  lecture_data();
  envoi_msg();

// Pause entre 2mesures...
//delay(3000);
//delay(1000);
// pour 10secondes de délai
//delay(20000);
// on ajoute 20sec

//delay(90054);
// on ajoute 90sec --> lecture toutes les 2min
//delay(3000); // ajustement pour atteindre 2min

//delay(500);

//cpt=cpt+1;
//Serial.print("cpt=");
//Serial.println(cpt);

//if (cpt>1){
//  //Serial.println("Fin...");
//  while (1) {}
//}

//  Sleep for XX minutes
LowPower.sleep(SLEEPTIME);

}


void lecture_data() {
  //Serial.println("Loop Readings:...");


//  Temperature #A capteur interne boitier
  ds.reset();
  addr[0]=16;
  addr[1]=35;
  addr[2]=171;
  addr[3]=20;
  addr[4]=3;
  addr[5]=8;
  addr[6]=0;
  addr[7]=243;
  
 // the first ROM byte indicates which chip
  switch (addr[0]) {
    case 0x10:
      ////Serial.println("  Chip = DS18S20");  // or old DS1820
      type_s = 1;
      break;
    case 0x28:
      ////Serial.println("  Chip = DS18B20");
      type_s = 0;
      break;
    case 0x22:
      ////Serial.println("  Chip = DS1822");
      type_s = 0;
      break;
    default:
      //Serial.println("Device is not a DS18x20 family device.");
      return;
  } 
  ds.select(addr);
  ds.write(0x44, 1);        // start conversion, with parasite power on at the end
  
  delay(1000);     // maybe 750ms is enough, maybe not
  // we might do a ds.depower() here, but the reset will take care of it.
  
  present = ds.reset();
  ds.select(addr);    
  ds.write(0xBE);         // Read Scratchpad

  for ( i = 0; i < 9; i++) {           // we need 9 bytes
    data[i] = ds.read();
  }


  // Convert the data to actual temperature
  // because the result is a 16 bit signed integer, it should
  // be stored to an "int16_t" type, which is always 16 bits
  // even when compiled on a 32 bit processor.
  int16_t raw = (data[1] << 8) | data[0];
  if (type_s) {
    raw = raw << 3; // 9 bit resolution default
    if (data[7] == 0x10) {
      // "count remain" gives full 12 bit resolution
      raw = (raw & 0xFFF0) + 12 - data[6];
    }
  } else {
    byte cfg = (data[4] & 0x60);
    // at lower res, the low bits are undefined, so let's zero them
    if (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
    else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
    else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
    //// default is 12 bit resolution, 750 ms conversion time
  }
  Tc_100_A = (float)raw / 16.0;
  //Serial.print("Temp=");
  //Serial.println(Tc_100_A);

/*
//  Temperature #B  capteur volant
  ds.reset();
  addr[0]=16;
  addr[1]=228;
  addr[2]=224;
  addr[3]=37;
  addr[4]=1;
  addr[5]=8;
  addr[6]=0;
  addr[7]=248;
 // the first ROM byte indicates which chip
  switch (addr[0]) {
    case 0x10:
      ////Serial.println("  Chip = DS18S20");  // or old DS1820
      type_s = 1;
      break;
    case 0x28:
      ////Serial.println("  Chip = DS18B20");
      type_s = 0;
      break;
    case 0x22:
      ////Serial.println("  Chip = DS1822");
      type_s = 0;
      break;
    default:
      //Serial.println("Device is not a DS18x20 family device.");
      return;
  } 
  ds.select(addr);
  ds.write(0x44, 1);        // start conversion, with parasite power on at the end
  
  delay(1000);     // maybe 750ms is enough, maybe not
  // we might do a ds.depower() here, but the reset will take care of it.
  
  present = ds.reset();
  ds.select(addr);    
  ds.write(0xBE);         // Read Scratchpad

  for ( i = 0; i < 9; i++) {           // we need 9 bytes
    data[i] = ds.read();
  }


  // Convert the data to actual temperature
  // because the result is a 16 bit signed integer, it should
  // be stored to an "int16_t" type, which is always 16 bits
  // even when compiled on a 32 bit processor.
  raw = (data[1] << 8) | data[0];
  if (type_s) {
    raw = raw << 3; // 9 bit resolution default
    if (data[7] == 0x10) {
      // "count remain" gives full 12 bit resolution
      raw = (raw & 0xFFF0) + 12 - data[6];
    }
  } else {
    byte cfg = (data[4] & 0x60);
    // at lower res, the low bits are undefined, so let's zero them
    if (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
    else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
    else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
    //// default is 12 bit resolution, 750 ms conversion time
  }
  
  Tc_100_B = (float)raw / 16.0;

*/

    Tc_100_B = SigFox.internalTemperature();

    scaleA.power_up();
    poidsD=1*scaleA.get_units(10);
    scaleA.power_down();              // put the ADC in sleep mode
  
    //Serial.println("------");
    //Serial.print("Poids="); 
    //Serial.print(poidsD);
    //Serial.println("g");

  
      //Read data and store it to variables hum and temp from DHT22 on pin #D
    hum = dht.readHumidity();
    temp= dht.readTemperature();
    //Serial.print("temp="); 
    //Serial.print(temp); 
    //Serial.print("°C hum="); 
    //Serial.print(hum);
    //Serial.println("%");


    lire_batterie();
    //Serial.print("Batt.=");
    //Serial.print(Vf);
    //Serial.println("%");
    
}

void envoi_msg() {



// Preparation pour envoi Sigfox...


poidsD=(poidsD/1000.0);
double decimalpoids;
double entpoids;
entpoids=(int)poidsD;
decimalpoids=(poidsD - ((int) poidsD))*1000.0;
if (decimalpoids<0) {
  entpoids=entpoids-1;
  poidsD=poidsD-1;
  decimalpoids=(poidsD - ((int) poidsD))*1000.0;
}
 
  // on initialise avec des valeurs reconnnaissables
  //__builtin_bswap16 == pour passer de little_endian en big_endian   (most significant bit first)
  msgsgfox.t_arduino =  __builtin_bswap16((Tc_100_A+30)*10);
  msgsgfox.t_int1 =  __builtin_bswap16((Tc_100_B+30)*10);
  msgsgfox.t_int2 =  __builtin_bswap16((temp+30)*10);
  msgsgfox.hum =  __builtin_bswap16(hum*10.0);
  msgsgfox.poids_kg = (entpoids);
  msgsgfox.poids_g = __builtin_bswap16(decimalpoids+1);
  msgsgfox.bat = Vf;

  //Serial.print("t_arduino=");
  //Serial.println(msgsgfox.t_arduino); 
  //Serial.print("t_int1=");
  //Serial.println(msgsgfox.t_int1); 
  //Serial.print("t_int2=");
  //Serial.println(msgsgfox.t_int2); 
  //Serial.print("poids_kg=");
  //Serial.println(msgsgfox.poids_kg); 
  //Serial.print("poid_g=");
  //Serial.println(msgsgfox.poids_g);
  //Serial.print("bat=");
  //Serial.println(msgsgfox.bat); 
   
  transmission_sigfox();
}


void transmission_sigfox() {
    // Clears all pending interrupts
  
  // Start the module
  SigFox.begin();
  // Wait at least 30ms after first configuration (100ms before)
  delay(100);
  SigFox.status();
  delay(1);

  SigFox.beginPacket();
  SigFox.write((uint8_t*)&msgsgfox, 12);

  int ret;
  ret= SigFox.endPacket();  // send buffer to SIGFOX network
  if (ret > 0) {
    //Serial.println("No transmission");
  } else {
    //Serial.println("Transmission ok");
  }
  //Serial.println(SigFox.status(SIGFOX));
  //Serial.println(SigFox.status(ATMEL));  
  SigFox.end();
}

void sendString(String str) {
  // Start the module
  SigFox.begin();
  // Wait at least 30mS after first configuration (100mS before)
  delay(100);
  // Clears all pending interrupts
  SigFox.status();
  delay(1);

  SigFox.beginPacket();
  SigFox.print(str);

  int ret = SigFox.endPacket();  // send buffer to SIGFOX network
  if (ret > 0) {
    //Serial.println("No transmission");
  } else {
    //Serial.println("Transmission ok");
  }

  //Serial.println(SigFox.status(SIGFOX));
  //Serial.println(SigFox.status(ATMEL));
  SigFox.end();
}

void sendStringAndGetResponse(String str) {
  // Start the module
  SigFox.begin();
  // Wait at least 30mS after first configuration (100mS before)
  delay(100);
  // Clears all pending interrupts
  SigFox.status();
  delay(1);

  SigFox.beginPacket();
  SigFox.print(str);

  int ret = SigFox.endPacket(true);  // send buffer to SIGFOX network and wait for a response
  if (ret > 0) {
    //Serial.println("No transmission");
  } else {
    //Serial.println("Transmission ok");
  }

  //Serial.println(SigFox.status(SIGFOX));
  //Serial.println(SigFox.status(ATMEL));

  if (SigFox.parsePacket()) {
    //Serial.println("Response from server:");
    while (SigFox.available()) {
      //Serial.print("0x");
      //Serial.println(SigFox.read(), HEX);
    }
  } else {
    //Serial.println("Could not get any response from the server");
    //Serial.println("Check the SigFox coverage in your area");
    //Serial.println("If you are indoor, check the 20dB coverage or move near a window");
  }
  //Serial.println();

  SigFox.end();
}


void reboot() {
        NVIC_SystemReset();
        while (1) ;
}


void lire_batterie() {
  
int i(0);
int sensorValue(0);
Vf=0;
for ( i = 0; i < 10; i++) {           // repeter 10fois
    sensorValue=analogRead(ADC_BATTERY);
    Vf = Vf+sensorValue * (100 / 4095.0);
}
Vf=Vf/10;
}
